#!/usr/bin/env python
import sys
import os
import os.path
import numpy as np

input_dir = sys.argv[1]
output_dir = sys.argv[2]

submit_dir = os.path.join(input_dir, 'res')
truth_dir = os.path.join(input_dir, 'ref')

output_filename = os.path.join(output_dir, 'scores.txt')
output_file = open(output_filename, 'wb')

if not os.path.isdir(submit_dir):
    print("%s doesn't exist" % submit_dir)

if os.path.isdir(submit_dir) and os.path.isdir(truth_dir):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    right_pd = 0.0
    total_pt = 0.0
    tp = np.zeros((10,))
    fp = np.zeros((10,))
    fn = np.zeros((10,))
    iou = np.zeros((10,))
    auroc = np.zeros((11,))
    ood = np.zeros((11,))
    normal = np.zeros((11,))
    right_ood = np.zeros((101,11))
    wrong_ood = np.zeros((101,11))
    thres = np.arange(101).astype(np.float32)/100
    TPR = np.zeros_like(right_ood)
    FPR = np.zeros_like(right_ood)
    for i in range(301):
        filename = str(i).zfill(6)
        gt = np.fromfile(os.path.join(truth_dir, filename + '.label'), dtype=np.uint32) & 0xffff
        pd = np.fromfile(os.path.join(submit_dir,'predictions', filename + '.label'), dtype=np.uint32) & 0xffff
        conf = np.fromfile(os.path.join(submit_dir,'confidence', filename + '.conf'), dtype=np.float32)

        # acc
        total_pt += np.sum((gt>0)*(gt<10))
        right_pd += np.sum(gt==pd)

        # iou
        for i in range(1,10):
            tp[i] += np.sum((gt==i)*(pd==i))
            fp[i] += np.sum((gt>0)*(gt<10)*(gt!=i)*(pd==i))
            fn[i] += np.sum((gt==i)*(pd!=i))

        # AUROC
        for C in range(1,11):
            if C == 10:
                pred_mask = (gt != 0)
            else:
                pred_mask = (gt != 0) * (pd == C)
            ood_mask = (gt == 10) * pred_mask
            normal_mask = (gt > 0) * (gt < 10) * pred_mask
            normal[C] += np.sum(normal_mask)
            ood[C] += np.sum(ood_mask)
            for ti in range(101):
                t = thres[ti]
                mask = (conf > t)
                right_ood[ti, C] += np.sum(normal_mask * mask)
                wrong_ood[ti, C] += np.sum(ood_mask * mask)

    for i in range(1, 10):
        iou[i] = tp[i] / (tp[i] + fp[i] + fn[i])
    miou = np.average(iou[1:])
    for C in range(1, 11):
        if normal[C]==0: # the predicted class will always be OOD
            auroc[C] = 0
            continue

        elif ood[C]==0: # the predicted class will always be ID
            auroc[C] = 1
            continue

        for ti in range(101):
            if ti==0:
                TPR[ti, C] = 1
                FPR[ti, C] = 1
            elif ti==100:
                TPR[ti, C] = 0
                FPR[ti, C] = 0
            else:
                TPR[ti, C] = right_ood[ti,C] / normal[C]
                FPR[ti, C] = wrong_ood[ti,C] / ood[C]
        for i in range(len(FPR) - 1):
            auroc[C] -= (TPR[i,C] + TPR[i + 1,C]) * (FPR[i + 1,C] - FPR[i,C]) / 2

    np.set_printoptions(threshold=1200)
    print("TPR")
    print(TPR)
    print("FPR")
    print(FPR)

    output_file.write("accuracy:%s\n"%(right_pd/total_pt))
    output_file.write("auroc:%s\n" % (auroc[10]))
    for i in range(1, 10):
        output_file.write("iou%s:%s\n" % (i,iou[i]))
    for i in range(1, 10):
        output_file.write("auroc%s:%s\n" % (i,auroc[i]))
    output_file.write("miou:%s\n" % (miou))
    output_file.close()
